@@ -6,6 +6,7 @@ from typing import Type, cast, List
 
 import pytextrank  # noqa: F401
 import spacy
+from collections_extended import RangeMap
 from pydantic import BaseModel, Field
 from pymultirole_plugins.v1.formatter import FormatterParameters, FormatterBase
 from pymultirole_plugins.v1.processor import ProcessorBase, ProcessorParameters
@@ -13,6 +14,7 @@ from pymultirole_plugins.v1.schema import Document
 from spacy.cli.download import download_model, get_compatibility, get_version
 from spacy.errors import OLD_MODEL_SHORTCUTS
 from spacy.language import Language
+from spacy.tokens import Doc
 from starlette.responses import Response, PlainTextResponse
 from wasabi import msg
 
@@ -28,6 +30,7 @@ class TextRankSummarizerParameters(FormatterParameters):
                       description="Name of the 2-letter language of the documents")
     num_sentences: float = Field(0.25, description="""Number of sentences of the summary:<br/>
         <li>If float in the range [0.0, 1.0], then consider num_sentences as a percentage of the original number of sentences of the document.""")
+    num_phrases: int = Field(10, description="Maximum number of top-ranked phrases to use in the distance vectors.")
 
 
 class TextRankSummarizerFormatter(FormatterBase, ProcessorBase, ABC):
@@ -35,6 +38,29 @@ class TextRankSummarizerFormatter(FormatterBase, ProcessorBase, ABC):
     """
 
     # cache_dir = os.path.join(xdg_cache_home, 'trankit')
+    def _check_sentences(self, nlp, document: Document):
+        doc: Doc = None
+        if not document.sentences:
+            doc = nlp(document.text)
+        else:
+            textrank = nlp.get_pipe("textrank")
+            parser = nlp.get_pipe("parser")
+            with nlp.select_pipes(disable=["parser", "textrank"]):
+                doc = nlp(document.text)
+                sent_map = RangeMap()
+                for i, s in enumerate(document.sentences):
+                    if s.end > s.start:
+                        sent_map[s.start:s.end] = i
+                current = -1
+                for token in doc:
+                    if token.idx in sent_map:
+                        idsent = sent_map[token.idx]
+                        if idsent != current:
+                            token.is_sent_start = True
+                            current = idsent
+                if doc.has_annotation("SENT_START"):
+                    doc = textrank(parser(doc))
+        return doc
 
     def _summarize(self, document: Document, parameters: FormatterParameters) -> str:
         def int_float(v: float):
@@ -45,9 +71,8 @@ class TextRankSummarizerFormatter(FormatterBase, ProcessorBase, ABC):
         params: TextRankSummarizerParameters = \
             cast(TextRankSummarizerParameters, parameters)
         nlp = get_nlp(params.lang)
-        doc = nlp(document.text)
+        doc = self._check_sentences(nlp, document)
         sent_bounds = [[s.start, s.end, set([])] for s in doc.sents]
-
         num_sentences = int_float(params.num_sentences)
         limit_sentences = round(len(sent_bounds) * num_sentences) if isinstance(num_sentences, float) else num_sentences
         phrase_id = 0
@@ -60,8 +85,8 @@ class TextRankSummarizerFormatter(FormatterBase, ProcessorBase, ABC):
                         sent_vector.add(phrase_id)
                         break
             phrase_id += 1
-            # if phrase_id == limit_phrases:
-            #     break
+            if phrase_id == params.num_phrases:
+                break
 
         sum_ranks = sum(unit_vector)
 
